@@ -1,9 +1,10 @@
-__version__ = "0.1.2"
+__version__ = "0.1.3"
 __author__ = "Sadiq Rahmati"
+
 import logging
 
 from .ticker import Ticker
-from .tse import TSE
+from .market import Market
 from .tse_scrapper import TSEScrapper
 
-logging.getLogger(__name__).addHandler(logging.NullHandler())
+logging.getLogger("tfinance").addHandler(logging.NullHandler())
