@@ -1,16 +1,16 @@
 import logging
 
-from .tse import TSE
+from .market import Market
 
 
 class Ticker:
 
-    def __init__(self, ticker):
+    def __init__(self, **kwargs):
         self.logger = logging.getLogger(__name__)
-        tse = TSE()
-        df = tse.tickers
-        self._info = df[df.ticker == ticker].iloc[0].to_dict()
-        self._history = tse.fetch_ticker_history(ticker=ticker)
+        market = Market()
+        df = market.fetch_tickers_filter_by(**kwargs)
+        self._info = df.iloc[0].to_dict()
+        self._history = market.fetch_history(**kwargs)
 
     @property
     def name(self):
